/**
 *
 */
package org.randombits.utils.text;

import org.randombits.utils.lang.API;

import java.nio.CharBuffer;

/**
 * Compares digit chunks of two text values. Intended to be used with
 * the {@link TextComparator}.
 */
@API
public class DigitProcessor implements TextProcessor {

    private Long getValue( CharBuffer text ) {
        TextBuffer out = new TextBuffer();

        while ( text.hasRemaining() && isDigit( text.charAt( 0 ) ) )
            out.add( text.get() );

        if ( out.length() == 0 )
            throw new IllegalArgumentException( "Expected digit: " + text );

        return Long.valueOf( out.toString() );
    }

    private boolean isDigit( char c ) {
        return Character.isDigit( c );
    }

    @Override
    public boolean canProcess( CharBuffer text ) {
        return text.hasRemaining() && isDigit( text.get() );
    }

    @Override
    public int compare( CharBuffer text1, CharBuffer text2 ) {
        Long v1 = getValue( text1 );
        Long v2 = getValue( text2 );
        return v1.compareTo( v2 );
    }
}