/**
 * 
 */
package org.randombits.utils.text;

import org.randombits.utils.lang.API;

import java.nio.CharBuffer;
import java.text.Collator;

/**
 * Compares 'letter' chunks in two Strings, as defined by {@link Character#isLetter(char)}.
 * This is intended to be used with {@link TextComparator}.
 */
@API
public class LetterProcessor implements TextProcessor {

    private Collator collator;

    @API
    public LetterProcessor() {
        this( null );
    }

    @API
    public LetterProcessor( Collator collator ) {
        this.collator = collator;
    }

    private boolean isLetter( char ch ) {
        return Character.isLetter( ch );
    }

    @Override
    public boolean canProcess( CharBuffer text ) {
        return text.hasRemaining() && isLetter( text.get() );
    }

    @Override
    public int compare( CharBuffer text1, CharBuffer text2 ) {
        TextBuffer v1 = getLetters( text1 );
        TextBuffer v2 = getLetters( text2 );

        if ( collator == null )
            return v1.compareTo( v2 );
        else
            return collator.compare( v1.toString(), v2.toString() );
    }

    private TextBuffer getLetters( CharBuffer text ) {
        TextBuffer out = new TextBuffer();
        while ( text.hasRemaining() && isLetter( text.charAt( 0 ) ) )
            out.add( text.get() );

        if ( out.length() == 0 )
            throw new IllegalArgumentException( "Expected letter: " + text );

        return out;
    }
}
