package org.randombits.utils.text;

import org.randombits.utils.lang.API;

import java.util.BitSet;
import java.util.regex.Pattern;

@API
public final class PatternUtils {
    private static final BitSet RESERVED = new BitSet( Character.MAX_VALUE );
    static {
        RESERVED.set( '\\' );
        RESERVED.set( '.' );
        RESERVED.set( '{' );
        RESERVED.set( '}' );
        RESERVED.set( '[' );
        RESERVED.set( ']' );
        RESERVED.set( '(' );
        RESERVED.set( ')' );
        RESERVED.set( '*' );
        RESERVED.set( '?' );
        RESERVED.set( '+' );
        RESERVED.set( ':' );
        RESERVED.set( '^' );
        RESERVED.set( '$' );
        RESERVED.set( '|' );
    }
    
    private PatternUtils() {
    }

    /**
     * Compiles a pattern based on the common simple matching syntax, with "?"
     * being a single character and "*" being multiple characters.
     * 
     * @param pattern
     *            The pattern value.
     * @return The compiled {@link java.util.regex.Pattern}.
     */
    @API
    public static Pattern compileSimple( String pattern ) {
        String expanded = expandSimple( pattern );
        return Pattern.compile( expanded );
    }

    private static String expandSimple( String pattern ) {
        StringBuilder expanded = new StringBuilder();
        char[] chars = pattern.toCharArray();

        for ( char aChar : chars ) {
            if ( RESERVED.get( aChar ) ) {
                if ( aChar == '?' )
                    expanded.append( ".{1}" );
                else if ( aChar == '*' )
                    expanded.append( ".*?" );
                else
                    // just escape it.
                    expanded.append( '\\' ).append( aChar );
            } else {
                expanded.append( aChar );
            }
        }

        return expanded.toString();
    }
}
