/**
 * 
 */
package org.randombits.utils.text;

import org.randombits.utils.lang.API;

import java.nio.CharBuffer;

/**
 * Compares 'symbol' chunks of two strings. These are sequences of characters
 * which are neither letters or digits. This is intended to be used with the
 * {@link TextComparator} class.
 */
@API
public class SymbolProcessor implements TextProcessor {

    @Override
    public boolean canProcess( CharBuffer text ) {
        return text.hasRemaining() && isSymbol( text.get() );
    }

    private boolean isSymbol( char ch ) {
        return !Character.isDigit( ch ) && !Character.isLetter( ch );
    }

    @Override
    public int compare( CharBuffer text1, CharBuffer text2 ) {
        TextBuffer s1 = getSymbols( text1 );
        TextBuffer s2 = getSymbols( text2 );
        return s1.compareTo( s2 );
    }

    private TextBuffer getSymbols( CharBuffer text ) {
        TextBuffer out = new TextBuffer();
        while ( text.hasRemaining() && isSymbol( text.charAt( 0 ) ) )
            out.add( text.get() );

        if ( out.length() == 0 )
            throw new IllegalArgumentException( "Expected symbol: " + text );
        return out;
    }

}