package org.randombits.utils.text;

import org.randombits.utils.lang.API;

import java.nio.CharBuffer;

/**
 * A common interface for classes which can process particular subsets of characters.
 */
@API
public interface TextProcessor {

    /**
     * Check if the processor can handle the text at its current position.
     * The text buffer will be marked prior to calling this method and
     * rewound afterwards, so implementations can process sections of the
     * text safely.
     *
     * @param text the text buffer.
     * @return <code>true</code> if this processor can handle the current
     *         text.
     */
    @API
    public boolean canProcess( CharBuffer text );

    /**
     * Compare two text buffers which have both passed the
     * {@link #canProcess(java.nio.CharBuffer)} method for this processor.
     *
     * @param text1 The first text buffer.
     * @param text2 The second text buffer.
     * @return the comparison value for the two buffers.
     * @throws IllegalArgumentException if either of the text buffers does not match the expected
     *                                  value.
     */
    @API
    public int compare( CharBuffer text1, CharBuffer text2 );
}
